import { shareReplay, map, switchMap, catchError, firstValueFrom } from 'rxjs';
import * as i0 from '@angular/core';
import { inject, Injector, Injectable } from '@angular/core';
import { Exception } from '@angularity/core';
import { HttpContextToken, HttpClient, HttpContext } from '@angular/common/http';
import { pendingUntilEvent } from '@angular/core/rxjs-interop';
import { HTTP_SERVER_CACHE_KEY } from '@angularity/core/http';

/**
 * Type-helper for creating `ConfigFileDefinition` objects.
 */
const defineConfigFile = (def) => def;

/**
 * Decorator for `ConfigFileLoader`
 * that caches the loaded configuration objects of each definition.
 *
 * One `ConfigFileDefinition` object would cause only one load operation.
 */
class CacheConfigFiles {
    #cache = new Map();
    constructor(kernel) {
        this.kernel = kernel;
    }
    load(def) {
        const cached = this.#cache.get(def);
        if (cached)
            return cached;
        const result$ = this.kernel.load(def).pipe(shareReplay(1));
        this.#cache.set(def, result$);
        return result$;
    }
}

/**
 * `HttpContextToken` that will be supplied a value in an endpoint request
 * sent by `HttpClientConfigFileLoader`.
 */
const CONFIG_FILE_REQUEST_CONTEXT = new HttpContextToken(() => null);
/**
 * Implementation of `ConfigFileLoader`
 * based on Angular's built-in `HttpClient`.
 *
 * `CONFIG_FILE_REQUEST_CONTEXT` will be supplied in the request context.
 */
class HttpClientConfigFileLoader {
    constructor() {
        this.httpClient = inject(HttpClient);
        this.injector = inject(Injector);
    }
    load(def) {
        const parser = this.injector.get(def.parser);
        const validator = this.injector.get(def.validator);
        const context = new HttpContext().set(CONFIG_FILE_REQUEST_CONTEXT, { def });
        return this.fetch(def.path, context).pipe(map((res) => {
            if (!res)
                throw new ConfigFileNotFoundException(def.path);
            return res;
        }), switchMap(async (raw) => {
            const parsed = await parser.parse(raw);
            await validator.validate(def.schema, parsed);
            return parsed;
        }), pendingUntilEvent(this.injector));
    }
    fetch(path, context) {
        return this.httpClient
            .get(path, { responseType: 'text', context })
            .pipe(catchError(() => [null]));
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: HttpClientConfigFileLoader, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: HttpClientConfigFileLoader, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: HttpClientConfigFileLoader, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root',
                }]
        }] });

/**
 * Service for loading configuration files.
 *
 * @remarks
 * By default, uses `HttpClientConfigFileLoader`
 * decorated with `CacheConfigFiles`.
 */
class ConfigFileLoader {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: ConfigFileLoader, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: ConfigFileLoader, providedIn: 'root', useFactory: () => new CacheConfigFiles(inject(HttpClientConfigFileLoader)) }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: ConfigFileLoader, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root',
                    useFactory: () => new CacheConfigFiles(inject(HttpClientConfigFileLoader)),
                }]
        }] });
class ConfigFileNotFoundException extends Exception {
}

/**
 * Load the target config file and return a promise
 * of its parsed content.
 * The promise may be rejected if the loading fails.
 */
function useConfigFile(def, loader = inject(ConfigFileLoader)) {
    return firstValueFrom(loader.load(def));
}

/**
 * HTTP interceptor that set a value to `HTTP_SERVER_CACHE_KEY` if:
 * - the `CONFIG_FILE_REQUEST_CONTEXT` is present, and
 * - the config definition has an ID.
 *
 * The value of the cache key is determined solely by the ID from
 * the config definition.
 *
 * @remarks Requires `httpServerCacheInterceptor`
 */
const httpServerCacheConfigFilesInterceptor = (req, next) => {
    const context = req.context.get(CONFIG_FILE_REQUEST_CONTEXT);
    if (!context)
        return next(req);
    if (!context.def.id)
        return next(req);
    const key = `config-files:${context.def.id}`;
    req = req.clone({ context: req.context.set(HTTP_SERVER_CACHE_KEY, key) });
    return next(req);
};

class ConfigFileParsingException extends Exception {
}

/**
 * Implementation of `ConfigFileParser` that simply wraps the raw string content.
 */
class TextParser {
    /**
     * Parse the raw string content of a configuration file and produce a
     * `{ value: string }` object.
     * @param value the raw string content
     * @returns `{ value: string }`
     */
    parse(value) {
        return { value };
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: TextParser, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: TextParser, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: TextParser, decorators: [{
            type: Injectable,
            args: [{ providedIn: 'root' }]
        }] });
/**
 * Implementation of `ConfigFileParser` that parses the string content as JSON,
 * via the built-in `JSON.parse` function.
 */
class JsonParser {
    /**
     * Parse the raw string content of a configuration file as JSON.
     */
    parse(raw) {
        try {
            return JSON.parse(raw);
        }
        catch (error) {
            throw new ConfigFileParsingException(String(error));
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: JsonParser, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: JsonParser, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: JsonParser, decorators: [{
            type: Injectable,
            args: [{ providedIn: 'root' }]
        }] });

class ConfigFileValidationException extends Exception {
}

/**
 * Implementation of `ConfigFileValidator` that does nothing,
 * i.e. the object is always considered valid.
 * @remarks The schema object is not used.
 */
class NoopValidator {
    validate() { }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: NoopValidator, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: NoopValidator, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: NoopValidator, decorators: [{
            type: Injectable,
            args: [{ providedIn: 'root' }]
        }] });

/**@deprecated */
const defineConfigFileMetadata = defineConfigFile;

/**
 * Generated bundle index. Do not edit.
 */

export { CONFIG_FILE_REQUEST_CONTEXT, CacheConfigFiles, ConfigFileLoader, ConfigFileNotFoundException, ConfigFileParsingException, ConfigFileValidationException, HttpClientConfigFileLoader, JsonParser, NoopValidator, TextParser, defineConfigFile, defineConfigFileMetadata, httpServerCacheConfigFilesInterceptor, useConfigFile };
//# sourceMappingURL=angularity-config-files.mjs.map
